searchData={"items":[{"type":"module","title":"GriffinSSG","doc":"Griffin is a Static Site Generator.\n\nGriffin reads Markdown files from disk and outputs HTML pages. A combination\nof Application level config, frontmatter attributes and layout files can be\nused to customize the output of each file and to build the page structure of\nthe website.\n\nEach input file can have a first segment called \"front matter\" that lists\nmetadata about the file contents. This front matter segment is delineated by\na sequence of characters `---` and the contents should be in YAML format.\nThe contents that follow can contain plain content or reference front matter\nattributes.\nHere is an example of a short Markdown file with some front matter attributes:\n\n```\n---\ntitle: \"Griffin Static Site Generator\"\ndraft: true\n---\n\n# Griffin Static Site Generator\nGriffin is an Elixir framework for building static websites.\n```","ref":"GriffinSSG.html"},{"type":"function","title":"GriffinSSG.list_pages/3","doc":"Lists all pages in a directory, returning metadata about each page.\nThe directory page is relative to the project root.","ref":"GriffinSSG.html#list_pages/3"},{"type":"function","title":"GriffinSSG.parse/3","doc":"Parses a file at the given path into two components: front matter and file content.","ref":"GriffinSSG.html#parse/3"},{"type":"function","title":"GriffinSSG.parse_string/1","doc":"Parses the string contents of a file into two components: front matter and file content.\n\nFront matter is an optional YAML snippet containing variables to be used in the content.\nThe content immediately follows the front matter and may reference front matter variables.\n\nReturns `{:ok, map()}` where map contains both the front matter and file content.","ref":"GriffinSSG.html#parse_string/1"},{"type":"function","title":"GriffinSSG.render/2","doc":"Renders a layout with a given content, front matter and assigns.\n\nThe layout is assumed to be a compiled EEx file or string, such that calling\n`Code.eval_quoted/2` on the layout will generate a correct result.","ref":"GriffinSSG.html#render/2"},{"type":"module","title":"GriffinSSG.Filesystem","doc":"Helper functions for handling basic file operations","ref":"GriffinSSG.Filesystem.html"},{"type":"function","title":"GriffinSSG.Filesystem.copy_all/2","doc":"Copies a list of files or directories into the `destination` directory.\nWhen successful, returns `{:ok, count}` where `count` is the number of copied files.\nif successful or `{:errors, list(string())}` when one or more copy\noperations failed.\nIn case of error, the `destination` directory will be left in a dirty state,\nand only some of the files might have been copied.","ref":"GriffinSSG.Filesystem.html#copy_all/2"},{"type":"function","title":"GriffinSSG.Filesystem.git_ignores/1","doc":"","ref":"GriffinSSG.Filesystem.html#git_ignores/1"},{"type":"function","title":"GriffinSSG.Filesystem.list_all/2","doc":"Lists all files from a path or wildcard.\nIf `filepath` is a path to a file, returns a list with only that filepath.\nIf `filepath` points to a directory, returns a list of all files inside\nthat directory and subdirectories.\nA list of ignored paths can be passed in to `opts`. The default value for\n`opts` is `[\".git\", \".elixir_ls\"]`","ref":"GriffinSSG.Filesystem.html#list_all/2"},{"type":"function","title":"GriffinSSG.Filesystem.output_filepath/3","doc":"Calculates the output path for a file.\nThe `filepath` is a contained within `input_dir`, which means that to\ncalculate the filepath in the output directory we need to see what is\nthe filepath relative to the `input_dir`. The end result is a concatenation\nof the `output_dir` with this relative path.","ref":"GriffinSSG.Filesystem.html#output_filepath/3"},{"type":"function","title":"GriffinSSG.Filesystem.search_directory/2","doc":"Searches a given path for files that have the selected extensions","ref":"GriffinSSG.Filesystem.html#search_directory/2"},{"type":"module","title":"GriffinSSG.Filesystem.Watcher","doc":"Module for non-named GenServer responsible for watching for changes.\nExecutes a generic callback when file changes are detected.\nUsed for the LiveReload HTTPServer that is launched as part of `mix grf.server`.","ref":"GriffinSSG.Filesystem.Watcher.html"},{"type":"function","title":"GriffinSSG.Filesystem.Watcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"GriffinSSG.Filesystem.Watcher.html#child_spec/1"},{"type":"function","title":"GriffinSSG.Filesystem.Watcher.init/1","doc":"","ref":"GriffinSSG.Filesystem.Watcher.html#init/1"},{"type":"function","title":"GriffinSSG.Filesystem.Watcher.start_link/1","doc":"","ref":"GriffinSSG.Filesystem.Watcher.html#start_link/1"},{"type":"module","title":"GriffinSSG.Layouts","doc":"Module responsible for compiling layouts. Stores compiled layouts in an ETS table.\nSupports nested layouts with a maximum nesting depth of 10.","ref":"GriffinSSG.Layouts.html"},{"type":"function","title":"GriffinSSG.Layouts.compile_layouts/1","doc":"","ref":"GriffinSSG.Layouts.html#compile_layouts/1"},{"type":"function","title":"GriffinSSG.Layouts.fallback_list_collection_layout/0","doc":"","ref":"GriffinSSG.Layouts.html#fallback_list_collection_layout/0"},{"type":"function","title":"GriffinSSG.Layouts.fallback_show_collection_layout/0","doc":"","ref":"GriffinSSG.Layouts.html#fallback_show_collection_layout/0"},{"type":"module","title":"GriffinSSG.Web.Plug","doc":"Defines a simple HTTP server with LiveReload. Spawned from the `grf.server` task.\nPlugLiveReload doesn't work within the context of a Plug.Builder, so we can't use\nPlug.Static to serve the files. Instead, we use Plug.Router and need to implement\nthe file serving logic, which is... unfortunate.","ref":"GriffinSSG.Web.Plug.html"},{"type":"function","title":"GriffinSSG.Web.Plug.call/2","doc":"","ref":"GriffinSSG.Web.Plug.html#call/2"},{"type":"function","title":"GriffinSSG.Web.Plug.init/1","doc":"","ref":"GriffinSSG.Web.Plug.html#init/1"},{"type":"task","title":"mix grf.build","doc":"Generates a static website from template and layout files.\n\n    $ mix grf.build [--input INPUT] [--output OUTPUT]\n\nTemplate and layout files will be read from INPUT directory and sub-folders,\nand output generated content to the OUTPUT directory.","ref":"Mix.Tasks.Grf.Build.html"},{"type":"task","title":"Options - mix grf.build","doc":"* `-in`, `--input` - the path to the input directory. Defaults to `src`.\n\n  * `-out`, `--output` - the directory where Griffin will write files to.\n    Defaults to `_site`\n\n  * `--layouts` - the directory where the layout and partials files are kept.\n    Defaults to `lib/layouts`.\n\n  * `--data` - the directory where global data files are stored.\n\n  * `--passthrough-copies` - comma separated list of directories or files to\n    copy directly to the output directory without processing.\n    Supports wildcard paths using `Path.wildcard/1` Useful for assets files.\n\n  * `--ignore` - comma separated list of directories or files to ignore\n    inside the input directory.\n\n  * `--config` - the path to the configuration file\n\n  * `--dry-run` - disables writing to the file system.\n    Useful for tests and debugging.\n\n  * `--quiet` - print minimal console output\n\n  * `--debug` - print additional debug information","ref":"Mix.Tasks.Grf.Build.html#module-options"},{"type":"task","title":"Passthrough copy - mix grf.build","doc":"Passthrough copy files are files that shouldn't be processed but simply\ncopied over to the output directory. This is useful for assets like images,\nfonts, JavaScript and CSS.\nA list of comma separated file or wildcard paths may be provided via the\n`--passthrough-copies` option. Here's an example:\n\n    $ mix grf.build --passthrough-copies=assets/js,fonts,images/*.{png,jpeg}\n\nThis command will copy all files in the `assets/js`, `fonts` and all PNG\nand JPEG images in the `images` directory over to the same path relative\nto the output directory. In the above example and assuming the default\n`_site` output directory, Griffin would copy files to `_site/assets/js`,\n`_site/fonts` and `_site/images` directories, respectively.\nWildcard paths are expanded by `Path.wildcard/1` and thus all options that\nit supports can be used to build wildcard paths.\n\n\n> #### About passthrough copy paths {: .neutral}\n>\n> The paths mentioned in this option are not relative to the input\n> directory, and instead are relative to the root of the Griffin project.","ref":"Mix.Tasks.Grf.Build.html#module-passthrough-copy"},{"type":"task","title":"Ignore files - mix grf.build","doc":"Griffin allows users to ignore specific files and/or directories via the\n`--ignore` option. This is useful for ignoring markdown files like readme\nand changelog files that might be in the source directory and that should\nnot be processed. A list of comma separated file wildcard paths can be\npassed using this option to ignore files or directories from processing.\n\nHere's an example:\n\n    $ mix grf.build --ignore=src/posts/drafts,src/README.md\n\nThis command will ignore all input files from the `src/posts/drafts`\ndirectory along with the `src/README.md` file.\nWildcard paths are expanded by `Path.wildcard/1` and thus all options that\nit supports can be used to build wildcard paths.\n\nThe paths mentioned in this option are not relative to the input\ndirectory, and instead are relative to the root of the Griffin project.\n\n> #### Default ignores {: .neutral}\n>\n> By default Griffin imports the ignores from your `.gitignore` file.","ref":"Mix.Tasks.Grf.Build.html#module-ignore-files"},{"type":"task","title":"Quiet option - mix grf.build","doc":"Griffin prints out information about files that it processed, including the\nrendering engine that processed the file (only `earmark` for now). For large\nprojects or other instances where users need minimal console output, there is\nthe `--quiet` option.","ref":"Mix.Tasks.Grf.Build.html#module-quiet-option"},{"type":"task","title":"Config file - mix grf.build","doc":"Griffin allows passing in ad-hoc configuration files through the `--config`\noption. This option accepts a path to a file that is then piped into\n`Code.eval_file/2`. Although this file can contain any Elixir code, it is\nexpected to return a map with the same configuration keys as those used by\nApplication environment. Here's an example `config.ex` file that returns a\nvalid Griffin config:\n\n```\n%{\n  # any other config key could be set here\n  input: \"custom_input_dir\",\n  output: \"custom_output_dir\"\n}\n```\n\nThis option simplifies configuration since it doesn't rely on Application\nenvironment, and it allows for better testing.","ref":"Mix.Tasks.Grf.Build.html#module-config-file"},{"type":"task","title":"Dry run - mix grf.build","doc":"If you're debugging an issue or just want to test Griffin out, you can use\nthe `--dry-run` option to run Griffin without writing to the file system.","ref":"Mix.Tasks.Grf.Build.html#module-dry-run"},{"type":"task","title":"Other options - mix grf.build","doc":"Griffin uses other configuration options that can be changed by setting\nspecific application environment keys under the `:griffin_ssg` application.\nThese other options include features that cannot be passed in as a single\nCLI option like hooks, shortcodes, filters, and more.","ref":"Mix.Tasks.Grf.Build.html#module-other-options"},{"type":"task","title":"Hooks - mix grf.build","doc":"Hooks are a way to allow user defined functions to be called at specific\nstages of the website generation process. The available hook events\nare:\n\n  - `before`, executed before the build process starts\n  - `after`, executed after Griffin finishes building.\n\nThe result from invoking these hooks is not checked.\n\nMultiple hooks of each kind can be set under\nthe `:hooks` configuration key like so:\n\n```\nconfig :griffin_ssg,\n  hooks: %{\n    before: [\n      fn { directories, run_mode, output_mode } ->\n        # Read more below about each type of event\n        :ok\n      end\n    ],\n    after: [\n      fn { directories, results, run_mode, output_mode } ->\n        # Read more below about each type of event\n        :ok\n      end\n    ]\n  }\n```\n\n#### Hook event arguments\n\nThese are the arguments that are passed in to the hook events:\n\n  * `directories`: a map containing the current project directories\n      * `directories.input` (defaults to `src`)\n      * `directories.output` (defaults to `_site`)\n      * `directories.layouts` (defaults to `lib/layouts`)\n  * `output_mode`: currently hardcoded to \"filesystem\"\n  * `run_mode`: currently hardcoded to \"build\"\n  * `results`: *(only avaiable on the `after` event)*. A list with the\n    processed Griffin output\n      * Each individual list item will have\n        `{ input_path, output_path, url, content }`","ref":"Mix.Tasks.Grf.Build.html#module-hooks"},{"type":"task","title":"Shortcodes - mix grf.build","doc":"Shortcodes are user definable functions that can be invoked inside layouts.\nThese functions enable easily reusable content. Shortcodes can be added under\nthe `shortcodes` configuration key. Here's an example shortcode for embedding\nYouTube videos:\n\n```\nconfig :griffin_ssg,\n  shortcodes: %{\n    youtube: fn slug ->\n      \"\"\"\n       \n       \n      \"\"\"\n    end\n  }\n```\n\nThis will create a `youtube` assigns variable that can be referenced in\nall layouts like so:\n\n```\n \n   Here's a classic YouTube video: \n  <%= @youtube.(\"dQw4w9WgXcQ\") %>\n \n```\n\nShortcodes can be defined with an arbitrary number of arguments and they are\nexpected to return content. They can reference variables or other shortcodes.\nWhen using shortcodes users can think about them as function components.","ref":"Mix.Tasks.Grf.Build.html#module-shortcodes"},{"type":"task","title":"Filters - mix grf.build","doc":"Filters are utility functions that can be used in layouts to transform and\ndata into a more presentable format.\n\nLike shortcodes, they are set in the application environment and they are\nprocessed into assigns variables that can be referred in all layouts.\n\nHere's an example of a layout that uses an `uppercase` filter:\n\n```\n <%= @username |> @uppercase.() %> \n```\n\nThis filter can be defined in the configuration file under the `:filters`\nconfiguration key:\n\n```\nconfig :griffin_ssg,\n  filters: %{\n    uppercase: &String.upcase/1\n  }\n```\n\n> #### Filters versus Shortcodes {: .neutral}\n>\n> Both filters and shortcodes are user defined functions that generate output\n> in some way. While shortcodes are meant to be convenient function\n> components that generate any sort of output, filters are typically designed\n> to be chained, so that the value returned from one filter is piped into the\n> next filter.","ref":"Mix.Tasks.Grf.Build.html#module-filters"},{"type":"task","title":"mix grf.new.page","doc":"Generates a new Markdown page with relevant front matter attributes.\n\n    $ mix grf.gen.page [--title TITLE] [--draft] PATH\n\nA Markdown file will be created with the name at the specified path\nwith the default metadata fields: `title`, `date` and `draft`.","ref":"Mix.Tasks.Grf.New.Page.html"},{"type":"task","title":"Options - mix grf.new.page","doc":"* `--title` - the title parameter in the frontmatter.\n    Defaults to the filename.\n\n  * `--draft` - marks the file as a draft in the frontmatter.","ref":"Mix.Tasks.Grf.New.Page.html#module-options"},{"type":"task","title":"mix grf.server","doc":"Starts up a local development server using plug_cowboy.\nThe local server watches for file changes and re-runs grf.build\non file change, but does not reload the file in the browser.\nThis server is NOT meant to be run in production.","ref":"Mix.Tasks.Grf.Server.html"},{"type":"function","title":"Mix.Tasks.Grf.Server.http_port/1","doc":"","ref":"Mix.Tasks.Grf.Server.html#http_port/1"},{"type":"extras","title":"Griffin and Competitors","doc":"# Griffin and Competitors\n\nGriffin is an Elixir [static site generator](https://www.cloudflare.com/learning/performance/static-site-generator/). There are many static site generators in use today, with highlight for **[11ty][11ty]** (JavaScript), **[hugo][hugo]** (Golang) and **[jekyll][jekyll]** (Ruby), which inspired the creation of Griffin.\n\nGriffin's goal is to allow developers to quickly build static sites using well known Elixir libraries and tools.","ref":"overview.html"},{"type":"extras","title":"Why Griffin? - Griffin and Competitors","doc":"- Griffin is [**fast.**](https://github.com/elixir-griffin/benchmarks)\n  It leverages Elixir and the BEAM to take advantage of all of your CPU cores to generate pages quickly, with most websites rendering under 1 second.\n\n- Griffin was made to **feel familiar to Elixir developers.**\n  It uses the same `mix` based installer as [Phoenix][phoenix], which will make\n  Elixir web developers feel right at home.\n\n- Griffin uses **independent template languages.**\n  Despite being an Elixir tool, content is written in Markdown with YAML\n  front matter to allow easier migrations to or from Griffin.\n\n- Griffin is **simple to use.**\n  We lower the barrier to entry by using sensible defaults\n  and minimizing the amount of configuration needed to set up a Griffin website.\n\n- Griffin is **built by the community.**\n  This is a new project that is still in active development.\n  Feel free to engage with us by [starring the project on Github][griffin-github],\n  contributing a pull-request or helping answer questions,\n  making sure to follow our [code of conduct][code-of-conduct].","ref":"overview.html#why-griffin"},{"type":"extras","title":"What about all the other Elixir tools? - Griffin and Competitors","doc":"Griffin is not the only tool of its kind written in Elixir. In fact, there are quite a few site generators written in Elixir, with some of the more relevant alternatives being [Serum][serum], [nimble_publisher][nimble_publisher] and [tableau][tableau]. All of these tools are worthy and each has a different take on supported formats, templating languages and installation methods. Griffin has its own opinionated take, which consists of:\n\n- Best in class documentation.\n- [`Mix`][mix] based installation and run scripts.\n- First class support for asset builders like [esbuild][esbuild] and [tailwind][tailwind].\n- Ease of use through a minimal set of features, intuitive design choices and sensible defaults.\n- Support for portable formats like Markdown for content and YAML for front matter.\n- Extensible templating engine that uses [`EEx`](https://hexdocs.pm/eex/EEx.html) by default.\n\nThese are the design choices that guide the development of Griffin and that make it unique from the other existing tools. If you read this far, keep going! Next up, read [Quick Start](quick_start.html)\n\n[phoenix]: https://hexdocs.pm/phoenix/\n[11ty]: https://www.11ty.dev/\n[hugo]: https://gohugo.io/\n[jekyll]: https://jekyllrb.com/\n[griffin-github]: https://github.com/elixir-griffin/griffin\n[code-of-conduct]: https://github.com/elixir-griffin/griffin/blob/main/CODE_OF_CONDUCT.md\n[esbuild]: https://hexdocs.pm/esbuild/Esbuild.html\n[tailwind]: https://hexdocs.pm/tailwind/Tailwind.html\n[mix]: https://hexdocs.pm/mix/Mix.html\n[serum]: https://dalgona.github.io/Serum/docs/index.html\n[nimble_publisher]: https://hexdocs.pm/nimble_publisher/NimblePublisher.html\n[tableau]: https://github.com/elixir-tools/tableau","ref":"overview.html#what-about-all-the-other-elixir-tools"},{"type":"extras","title":"Glossary","doc":"# Glossary\n\nThis page provides two lists of terms — one for [Griffin-specific terminology](#griffin-specific-terminology) that may be useful for when building out a project using Griffin, and one for [industry jargon](#industry-terms-and-buzzwords) that may be useful for understanding context.","ref":"glossary.html"},{"type":"extras","title":"Griffin-Specific Terminology - Glossary","doc":"#### Template\n\nA content file written in a format such as Markdown, HTML, HEEx or Liquid, which Griffin transforms into one or more pages in the built site. Templates can access [data](#data) exposed through the [data cascade](#data-cascade) with templating syntax.\n\n#### Layout\n\nA template which wraps around another template, typically to provide the scaffolding markup for content to sit in.\n\n[Read more about using layouts.](layouts.html)\n\n#### Data\n\nExposed via variables that can be used inside [templates](#template) and [layouts](#layout) using templating syntax. The data for a given template is aggregated through a process called the [data cascade](#data-cascade).\n\n#### Data Cascade\n\nGriffin's order of operations for evaluating all [data](#data) for any given [template](#template), and for resolving conflicts that arise. The data cascade follows the principle of colocation, so data defined broadly to apply to many templates will be overruled by data that targets the given template more specifically.\n\n<!-- [Read more about the data cascade.](data_cascade.html) -->\n\n#### Filter\n\nA function which can be used within templating syntax to transform [data](#data) into a more presentable format. Filters are typically designed to be chained, so that the value returned from one filter is piped into the next filter.\n\n[Read more about filters.](filters.html)\n\n#### Shortcode\n\nA function which can be used within templating syntax to inject content into templates. Shortcodes can take many arguments, and can be thought of as a templating approach to reusable markup.\n\n[Read more about shortcodes.](shortcodes.html)\n\n#### Collection\n\nAn array of [templates](#template), used to group similar content. Collections can be created by using [tags](collections.html#tag-syntax) or by calling the [collections API in the Griffin configuration](collections.html#advanced-custom-filtering-and-sorting).\n\n[Read more about collections.](collections.html)\n\n<!--","ref":"glossary.html#griffin-specific-terminology"},{"type":"extras","title":"Pagination - Glossary","doc":"A way to create pages by iterating over data. The same template is applied to each chunk of the paginated data.\n\n[Read more about pagination.](pagination.html) -->\n\n<!--","ref":"glossary.html#pagination"},{"type":"extras","title":"Plugin - Glossary","doc":"A portable, installable Griffin configuration which can add [data](#data), [filters](#filter), [shortcodes](#shortcode), and more to a project's setup.\n\n[Read more about plugins.](plugins.html) -->","ref":"glossary.html#plugin"},{"type":"extras","title":"Industry Terms and Buzzwords - Glossary","doc":"Our industry can be particularly bad about inventing words for things that already exist. Hopefully this page will help you navigate the labyrinth.\n\n#### Static Sites\n\nA static site is a group of generated HTML files. Content is built into the HTML files rather than using a dynamic back end language to generate the content on-the-fly. A dynamic site can appear static when you add caching rules to make the content stickier. A static site can appear dynamic when you run your build quickly and often.\n\n#### Data-Driven\n\nMake components and markup data-driven so that you don’t have a bunch of one-off copy-pasted HTML instances littered throughout your project.\n\n#### Zero Config\n\nZero config means that Griffin can run without any command line parameters or configuration files.\n\nWe’ve taken care to setup Griffin so that that running the stock  `grf.build` command uses sensible defaults. Lower the barrier to entry for that first project build to get up and running faster.\n\n#### Convention over Configuration Routing\n\nInstead of requiring a centralized configuration file for routing, `Griffin` routes map the file system, unless you override with a `permalink`.","ref":"glossary.html#industry-terms-and-buzzwords"},{"type":"extras","title":"Quick Start","doc":"# Quick Start\n\nGriffin requires Elixir version 1.14 or higher.\n\nYou can check if you have Elixir installed by running `iex --version` from a terminal. If the command is not found, you will need to [download and install Elixir](https://elixir-lang.org/install.html) before moving on.\n\n<!-- TODO record screencast building a simple website and put it here -->","ref":"quick_start.html"},{"type":"extras","title":"1. Create a new Elixir project - Quick Start","doc":"Create a new project using `mix new`:\n\n```console\nmix new blog\n```\n\nNow move into that directory with the `cd` command:\n\n```console\ncd blog\n```","ref":"quick_start.html#1-create-a-new-elixir-project"},{"type":"extras","title":"2. Add Griffin as a dependency - Quick Start","doc":"Edit the `mix.exs` file to include `griffin_ssg` as part of your dependencies:\n\n```elixir\n  defp deps do\n    [\n      # add the following line\n      # to your list of dependencies\n      {:griffin_ssg, \"~> 0.4\"},\n    ]\n  end\n```\n\n#### Fetch and install Griffin\n\nRun `mix deps.get` to fetch and install Griffin.","ref":"quick_start.html#2-add-griffin-as-a-dependency"},{"type":"extras","title":"3. Run Griffin - Quick Start","doc":"Let's use one of Griffin's scripts to test that the installation is working:\n\n```console\nmix grf.build\n```\n\nHere's what the terminal might look like after you've run this command:\n\n```console\n~/blog $ mix grf.build\nWrote 0 files in 0.03 seconds (v0.4.0)\n```\n\nIf you see `(v0.4.0)` that means you're running the latest version of Griffin. Note that Griffin didn't process any files -- this was expected, since we've not added templates yet.","ref":"quick_start.html#3-run-griffin"},{"type":"extras","title":"4. Create some templates - Quick Start","doc":"A _template_ is a content file written in a format such as Markdown, HTML or Liquid, which Griffin transforms into one or more pages when building our website. By default Griffin uses the files in `src` as input so first\n\n```console\ncd src\n```\n\nLet's create a couple of templates with the following commands:\n\n```console\necho '<!doctype html> My Cool Blog  Hello! ' > index.md\n```\n\n```console\necho '# Hello From Griffin' > README.md\n```\n\nYou can create these template files manually with any editor you like, just make sure to save them in your project folder and to use the right file extensions.\nNow that we have an HTML and a Markdown template, let's run Griffin again:\n\n```console\nmix grf.build\n```\n\nThe output may look like this:\n\n```console\n~/blog $ mix grf.build\nWriting _site/README/index.html from ./README.md (earmark)\nWriting _site/index.html from ./index.md (earmark)\nWrote 2 files in 0.06 seconds (v0.4.0)\n```\n\nWe’ve compiled our two content templates in the current directory into the output folder (`_site` is the default).","ref":"quick_start.html#4-create-some-templates"},{"type":"extras","title":"5. See the results - Quick Start","doc":"Let's use a different Griffin script to spin up a local HTTP server:\n\n```console\nmix grf.server\n```\n\nOpen `http://localhost:4000/` or `http://localhost:4000/README/` in your favorite web browser to see your Griffin site live! At the moment you still need to re-run `mix grf.build` every time you make changes to the templates, but we'll add a hot-reloading feature in an upcoming version.","ref":"quick_start.html#5-see-the-results"},{"type":"extras","title":"Core Concepts","doc":"# Core Concepts","ref":"core_concepts.html"},{"type":"extras","title":"Templates - Core Concepts","doc":"A template is a content file written in [Markdown](https://daringfireball.net/projects/markdown/basics).\nThis is where you'll write most of your content. Here's an example file:\n\n```markdown\n# People in space","ref":"core_concepts.html#templates"},{"type":"extras","title":"How many people are in space at this very moment? - Core Concepts","doc":"The answer is 10. Here are the ten brave souls:\n\n* Kayla Barron\n* Matthias Maurer\n* Thomas Marshburn\n* Raja Chari\n* Oleg Artemyev\n* Denis Matveev\n* Sergey Korsakov\n* Ye Guangfu\n* Wang Yaping\n* Zhai Zhigang\n```\n\nWhen building your website, Griffin takes files such as this and generates web pages from them using the [earmark](https://hex.pm/packages/earmark) library.","ref":"core_concepts.html#how-many-people-are-in-space-at-this-very-moment"},{"type":"extras","title":"Front Matter - Core Concepts","doc":"Front Matter is an **optional** header section written in [YAML](https://yaml.org/) that allow setting variables for a given page.\nIt is defined within two triple-dash (`---`) lines at the start of the template.\nHere's how we could add Front Matter to the previous template:\n\n```markdown\n---\ndate: 2023-04-05T21:51:54Z\ndraft: false\n---\n# People in space","ref":"core_concepts.html#front-matter"},{"type":"extras","title":"How many people are in space at this very moment? - Core Concepts","doc":"The answer is 10. Here are the ten brave souls:\n[...]\n```\n\nYou can use Front Matter variables inside templates using [Embedded Elixir](https://hexdocs.pm/eex/EEx.html) (EEx) like so:\n\n```elixir\n<%= @date %>\n```\n\nFront Matter is also used to configure multiple aspects of a page's output, such as the layout to render, the URL of the page, etc.","ref":"core_concepts.html#how-many-people-are-in-space-at-this-very-moment"},{"type":"extras","title":"Layouts - Core Concepts","doc":"Layouts are templates that wrap around your content.\nThey allow a clean separation between the structure and content of your website.\nOne layout may be used by many content pages, thus avoiding repetition of common snippets of code (e.g. navigation, footer).\n\nLayouts are typically [`.eex`](https://hexdocs.pm/eex/EEx.html) files and by default should live in the `lib/layouts` directory of your project.\n\n#### Usage\n\nWrite your HTML template to a file such as `lib/layouts/base.html.eex`, here's some example content:\n\n```html\n<!doctype html>\n \n   \n     \n     <%= @title %> \n     \n   \n   \n     \n       Home \n       Blog \n     \n     <%= @title %> \n     \n      <%= @content %>\n     \n     \n      &copy; <%= @author %>\n     \n   \n \n```\n\nWhen this layout is rendered, both `title` and `author` will come from the front matter of the template being rendered.\nThe value of the `content` variable will be the rendered output of the page being wrapped.\n\nIn your content pages, you'll need to specify in the front matter what layout to use:\n\n```markdown\n---\ntitle: Four score and seven years ago\nauthor: Abe Lincoln\nlayout: base\n---\n\nThe sixth sick sheik's sixth sheep's sick.\n```\n\nThe rendered output of this page will be:\n\n```html\n<!doctype html>\n \n   \n     \n     Four score and seven years ago \n     \n   \n   \n     \n       Home \n       Blog \n     \n     Four score and seven years ago \n     \n      The sixth sick sheik's sixth sheep's sick.\n     \n     \n      &copy; Abe Lincoln\n     \n   \n \n```\n\n#### Layouts can be nested\nLayouts can be nested by adding a Front Matter section and setting the `layout` variable:\n\n```html\n---\nlayout: base\n---\n I am a nested layout \n<%= @content %>\n```\n\nNesting layouts can be useful to reduce duplication of code. You can build a top level layout called `base`, which defines your core HTML with CSS and JS imports, and other, much simpler layouts for individual pages like `home`, `blog` or `about`.\n\n> #### Limit {: .tip}\n>\n> Griffin allows a maximum nesting level of 10 for layouts.","ref":"core_concepts.html#layouts"},{"type":"extras","title":"Partials - Core Concepts","doc":"Partials are layouts that cannot render `content` but are useful to extract common snippets of code. Partials typically live in the `lib/layouts/partials` directory.\n\nIn the previous example, you could extract the navigation snippet into a `lib/layouts/partials/navigation.html.eex` like so:\n\n```html\n \n   Home \n   Blog \n \n```\n\nLayouts can include partials using the `partials` variable.\n\nIf both the navigation and footer snippets were extracted to their own files, the template from the previous example would look like this:\n\n```html\n<!doctype html>\n \n   \n     \n     <%= @title %> \n     \n   \n   \n    <%= @partials.navigation %>\n     <%= @title %> \n     \n      <%= @content %>\n     \n    <%= @partials.footer %>\n   \n \n```\n\nPartials, like layouts, can reference front matter variables.","ref":"core_concepts.html#partials"},{"type":"extras","title":"Directory Structure - Core Concepts","doc":"A Griffin project for a blog might look like this:\n\n```shell\n.\n├── _site\n├── assets\n│   ├── css\n│   │   ├── app.min.css\n│   │   └── tailwind.min.css\n│   ├── img\n│   │   ├── one-monitor.jpeg\n│   │   └── six-monitors.png\n│   └── js\n│       ├── app.min.js\n│       └── jquery.min.css\n├── config\n│   └── config.exs\n├── lib\n│   └── layouts\n│       ├── base.html.eex\n│       ├── home.html.eex\n│       └── partials\n│           ├── header.html.eex\n│           └── footer.html.eex\n├── src\n│   ├── about.md\n│   ├── blog.md\n│   ├── blog\n│   │   ├── 2019-04-06-why-one-monitor-just-isnt-enough.md\n│   │   ├── 2020-04-06-two-monitors-is-good-but-three-is-better.md\n│   │   └── 2021-04-06-i-broke-my-desk-using-six-monitors-a-short-story.md\n│   └── index.md\n├── mix.exs\n└── mix.lock\n```\n\nGriffin can be configured to copy asset files via the `passthrough_copies`. You can use the following command to build the website:\n\n```\n    $ mix grf.build --passthrough-copies=assets\n```\n\nThis is what the output directory (`_site`) would look like:\n\n```shell\n.\n├── about.html\n│   └── index.html\n├── assets # all subdirectories copied\n├── blog\n│   ├── index.html\n│   ├── 2019-04-06-why-one-monitor-just-isnt-enough\n│   │   └── index.html\n│   ├── 2020-04-06-two-monitors-is-good-but-three-is-better\n│   │   └── index.html\n│   └── 2021-04-06-i-broke-my-desk-using-six-monitors-a-short-story\n│       └── index.html\n└── index.html\n```","ref":"core_concepts.html#directory-structure"},{"type":"extras","title":"Griffin directories - Core Concepts","doc":"By default these are the directories that are relevant for Griffin projects:\n\n| Directory     | Description                                                                                                                                                                                                                                                                                                     |\n|---------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|\n| `_site`       | The output directory where Griffin writes generated files to. After building your website, you can copy the contents of this directory into a hosting service like [Netlify](https://app.netlify.com/drop/), [GitHub Pages](https://pages.github.com/), [Cloudflare Pages](https://pages.cloudflare.com/), etc. |\n| `config`      | The configuration directory, usually containing a single `config.exs` file. These configuration files can add custom plugins, shortcodes, define pre/post-build hooks and more.                                                                                                                                 |\n| `lib/layouts` | The layouts directory. Usually contains EEx layouts and a optional `partials` subdirectory containing partial layouts.                                                                                                                                                                                          |\n| `src`         | The input directory. Griffin will read all Markdown templates inside this directory and generate pages from them if they are not marked as a draft in the front matter.                                                                                                                                         |","ref":"core_concepts.html#griffin-directories"},{"type":"extras","title":"Adding JS, CSS and Fonts","doc":"# Adding JS, CSS and Fonts\nCurrently there are two ways in which you can add assets to a Griffin project.","ref":"adding_js_css_fonts.html"},{"type":"extras","title":"1. Copying files using `passthrough-copies` - Adding JS, CSS and Fonts","doc":"In your configuration file, you can add a list of files or directories to be copied over to the output directory using the `passthrough_copies` configuration key:\n\n```elixir\n# config/config.exs\nconfig :griffin_ssg,\n  passthrough_copies: [\"styles.css\", \"bundle.js\"]\n```\n\nThen in your HTML you can reference these files:\n\n```html\n \n   \n     \n      \n   \n  <!-- rest of the page -->\n \n```\n\n> #### Passthrough-copies works with relative paths {: .info}\n>\n> Passthrough copies allows you to copy files from an assets folder to the output directory using relative paths.\n> If you have an `assets` folder in the root of your project and you want to copy it using passthrough copies,\n> Griffin will copy all the subfolders and files to ` /assets`.","ref":"adding_js_css_fonts.html#1-copying-files-using-passthrough-copies"},{"type":"extras","title":"2. Using Hooks - Adding JS, CSS and Fonts","doc":"As part of your configuration, you can write a hook function that copies all of the asset files into a directory inside the output folder.\nThis can be done in your configuration file, as usual:\n\n```elixir\n# config/config.exs\nconfig :griffin_ssg,\n  hooks: %{\n    before: [\n      fn _ ->\n        # Note: this function would also work as an `after` hook.\n        File.mkdir_p!(\"_site/assets\")\n        File.cp_r!(\"assets/vendor\", \"_site/assets/vendor\")\n        File.cp_r!(\"assets/img\", \"_site/assets\")\n      end\n    ]\n  }\n```","ref":"adding_js_css_fonts.html#2-using-hooks"},{"type":"extras","title":"Layouts","doc":"# Layouts\n\nLayouts are special templates that can be used to wrap other content.\nTo denote that a piece of content should be wrapped in a template, use the layout key in your front matter, like so:\n\n```markdown\n---\nlayout: cool_layout\ntitle: My Cool Griffin Blog Post\n---\n# <%= @title %>\n```\n\nThis will look for a `cool_layout.eex` EEx file in your *layouts* folder at `lib/layouts/cool_layout.eex`.\n\nNext, we need to create a `cool_layout.eex` file. It can contain any type of text, but here we’re using HTML:\n\n```html\n---\ntitle: My Griffin Blog\n---\n<!doctype html>\n \n   \n     \n     \n     <%= @title %> \n   \n   \n    <%= @content %>\n   \n \n```\n\nNote that the layout template will populate the `@content` data with the child template’s content.\n\nLayouts can contain their own front matter data! It’ll be merged with the content’s data on render. Content data takes precedence, if conflicting keys arise. Read more about how [Griffin merges data in what we call the Data Cascade](data_cascade.html).\n\nAll of this will output the following HTML content to `_site/some-page/index.html`:\n\n```html\n<!doctype html>\n \n   \n     \n     \n     My Cool Griffin Blog Post \n   \n   \n     My Cool Griffin Blog Post \n   \n \n```","ref":"layouts.html"},{"type":"extras","title":"Front matter data in layouts - Layouts","doc":"In [Griffin's Data Cascade](data_cascade.html), front matter data in your template is merged with Layout front matter data! All data is merged ahead of time so that you can mix and match variables in your content and layout templates interchangeably.\n\nFront matter data set in a content template takes priority over layout front matter! The closer to the content, the higher priority the data.\n\n#### Sources of data\n\nWhen the data is merged in the [Data Cascade](data_cascade.html), the order of priority for sources of data is (from highest priority to lowest):\n\n1. Computed Data\n1. Front Matter Data in a Template\n1. Template Data Files\n1. Directory Data Files\n1. Front Matter Data in Layouts\n1. Configuration Global Data\n1. Global Data Files","ref":"layouts.html#front-matter-data-in-layouts"},{"type":"extras","title":"Layout aliasing - Layouts","doc":"In your `config.exs` configuration file, you can a mapping of aliases for layouts using the `layout_aliases` configuration key:\n\n```elixir\nconfig :griffin_ssg,\n  layout_aliases: %{post: \"cool_layout.eex\"}\n```","ref":"layouts.html#layout-aliasing"},{"type":"extras","title":"Collections","doc":"# Collections","ref":"collections.html"},{"type":"extras","title":"What are collections? - Collections","doc":"A collection is a mechanism for grouping content pages. One common example of collections are `tags` in a blog,\nwhere aside from the user generated content there is a `/tags` page listing all of the tags\nand multiple `/tags/:tag` pages, one for each tag, grouping all of the posts that used that particular tag.\n\nGriffin allows users to define an arbitrary number of collections through the configuration file.\nFor a movie rating website, the following collections could be defined:\n\n```elixir\n# config/config.exs\nimport Config\n\nconfig :griffin_ssg,\n  collections: %{\n    cast: %{...},\n    directors: %{...},\n    production_companies: %{...},\n    year: %{...},\n    genres: %{...}\n  }\n```\n\nAfter setting a proper configuration for each collection, Griffin would be able to start grouping all of the pages according to the data available in the front matter. Using the example for a movie rating website, the following could be one possible template:\n\n```markdown\n# src/movies/the-hateful-eigth.md\n---\ntitle: The Hateful Eight\nyear: 2015\ndirectors: Quentin Tarantino\nposter: https://upload.wikimedia.org/wikipedia/en/d/d4/The_Hateful_Eight.jpg\ncast:\n  - Samuel L. Jackson\n  - Kurt Russell\n  - Jennifer Jason Leigh\n  - Walton Goggins\n  - Demián Bichir\n  - Tim Roth\n  - Michael Madsen\n  - Bruce Dern\n  - James Parks\n  - Channing Tatum\nproduction_companies:\n  - The Weinstein Company\n  - Shiny Penny\n  - FilmColony\n  - Double Feature Films\n  - Visiona Romantica, Inc.\ngenres:\n  - western\n  - mystery\n  - thriller\n---\nThe Hateful Eight (sometimes marketed as The H8ful Eight or The Hateful 8) is a\n2015 American Western mystery thriller film written and directed by Quentin Tarantino.\n[...]\n```\n\nAfter rendering each individual page, Griffin will generate collection pages for each `cast`, `directors`, `production_companies`, `year` and `genres`. Griffin will generate a listing page for all values of each collection as well as a listing page for all pages associated with a given collection value. Using the movie example from above, the following pages would be generated for `genres`:\n\n```text\n/genres/\n/genres/western/\n/genres/comedy/\n[...]\n```\n\nThe `/genres/` page would list and link to each individual genre (e.g. western, comedy, etc.) and the remaining pages would list all content pages associated with each genre.","ref":"collections.html#what-are-collections"},{"type":"extras","title":"Configuring a collection - Collections","doc":"Collections can be defined in the configuration file of your application under the `collections` configuration key. Each collection's definition is a name (`\"string\"` or `:atom`) and a map with its configuration parameters:\n\n```elixir\n# config/config.exs\nimport Config\n\nconfig :griffin_ssg,\n  collections: %{\n    \"albums\": %{\n      permalink: \"/movie/genres\",\n      list_layout: \"list_genres\",\n      show_layout: \"show_genre\"\n    }\n  }\n```\n\nCollections accept the following configuration parameters:\n\n* `permalink` - the path where the collection is written. Defaults to `/ `.\n* `list_layout` - the layout used to list all values for the collection (e.g. all movie genres)\n* `show_layout` - the layout used to list all pages associated with a given value (e.g. all western movies)","ref":"collections.html#configuring-a-collection"},{"type":"extras","title":"Collection list layout - Collections","doc":"The collection list layout (`list_layout`) is the layout used to list all of the values of a single collection. When rendering this layout, Griffin injects the following variables:\n\n* `@collection_name`, containing the name of the collection (e.g. `\"genres\"`)\n* `@collection_values`, containing a mapping of all pages associated with each collection value:\n```elixir\n%{\n    \"western\" => [%{url: \"/movie1/\"}, %{url: \"/movie2/\"}],\n    \"comedy\" => [%{url: \"/movie1/\"}]\n}\n```\n\nHere's an example of a basic list layout built for listing `genres`:\n\n```html\n<!DOCTYPE html>\n \n   \n     All Movie Genres \n   \n   \n     All Movie Genres \n     \n    <%= for {genre, _pages} <- @collection_values do %>\n        <%= genre %>  \n    <% end %>\n     \n   \n \n```","ref":"collections.html#collection-list-layout"},{"type":"extras","title":"Collection show layout - Collections","doc":"The collection show layout (`show_layout`) is the layout used to list content pages associated with a single collection value. When rendering this layout, Griffin injects the following variables:\n* `@collection_name`, containing the name of the collection (e.g. `:tags`)\n* `@collection_value`, containing the name of the collection value (e.g. `\"tag1\"`)\n* `@collection_value_pages`, containing a list of maps (e.g. `[%{url: \"/page1/\"}]`),\nwhere each map contains page information such as URL, title, description, date, input and output paths. Of these attributes, title and description can be `nil` if they are not set in the page's front matter.\n\nHere's an example of a basic show layout built for listing all pages with a given `tag`:\n\n```html\n<!DOCTYPE html>\n \n   \n     Pages tagged <%= @collection_value %> \n   \n   \n     Pages tagged `<%= @collection_value %>` \n     \n    <%= for page <- @collection_value_pages do %>\n        <%= page.data.title %>  \n    <% end %>\n     \n   \n \n```\n\n> #### Collection show layout is optional {: .info}\n>\n> If you don't wish to have pages for each collection value (e.g. `/genres/western`) you can design your own\n> custom `list_layout`, since Griffin injects all of the collection information (including the mappings from\n> values to pages) into this layout.","ref":"collections.html#collection-show-layout"},{"type":"extras","title":"Content Dates","doc":"# Content Dates","ref":"content_dates.html"},{"type":"extras","title":"Setting a Content Date in front matter - Content Dates","doc":"Add a `date` variable to your front matter to override the default date (last modified date) and customize how the file is sorted in a collection.\n\n```yaml\n---\ndate: 2023-04-25\n---\n```\n\nGriffin expects the date value to be a valid [ISO 8601][iso8601] date.","ref":"content_dates.html#setting-a-content-date-in-front-matter"},{"type":"extras","title":"Fallback value - Content Dates","doc":"When parsing content files, Griffin will attempt to read from the `date` front matter variable and convert it to a [ISO 8601][iso8601] date. If this conversion fails or the variable is undefined in the front matter, Griffin will instead use the file's last modified date (`ctime` in [File.Stat](https://hexdocs.pm/elixir/1.12/File.Stat.html))\n\n[iso8601]: https://en.wikipedia.org/wiki/ISO_8601","ref":"content_dates.html#fallback-value"},{"type":"extras","title":"Permalinks","doc":"# Permalinks\n\nYou can customize the default location of templates to the output directory using permalinks.\n\nHere are some examples of how it works by default (assuming your output directory is the default, `_site`):\n\n| Input                    | Output                     | Href              |\n|--------------------------|----------------------------|-------------------|\n| `index.eex`              | `index.html`               | `/`               |\n| `blog.eex`               | `blog/index.html`          | `/blog/`          |\n| `blog/somepost.md`       | `blog/somepost/index.html` | `/blog/somepost/` |\n| `blog/somepost/index.md` | `blog/somepost/index.html` | `/blog/somepost/` |\n\n#### Cool URIs don't change\nIf you're migrating to Griffin from some other piece of software which would generate different URIs for your existing content,\nyou can use `permalink` to make sure that your [Cool URIs don’t change](https://www.w3.org/Provider/Style/URI).","ref":"permalinks.html"},{"type":"extras","title":"Changing the output - Permalinks","doc":"To remap your template’s output to a different path than the default, use the permalink key in the template’s front matter. If a subdirectory does not exist, it will be created.\n\n```yaml\n---\npermalink: \"some-new-directory/subdirectory/content/index.html\"\n---\n```\n\nA template with this front matter will be written to `_site/some-new-directory/subdirectory/content/index.html`","ref":"permalinks.html#changing-the-output"},{"type":"extras","title":"Custom output formats - Permalinks","doc":"You can change the file extension in the permalink to output to any file type. For example, to generate a JSON search index to be used by popular search libraries:\n\n```eex\n---\npermalink: \"index.json\"\n---\n<%= Jason.encode!(@collections.all) %>\n```","ref":"permalinks.html#custom-output-formats"},{"type":"extras","title":"Data Cascade","doc":"# Data Cascade\n\nEach template is rendered with data that affects the output of the page. This data that is passed into each template is merged from multiple different sources before the template is rendered. The data is merged in a process we call the Data Cascade.","ref":"data_cascade.html"},{"type":"extras","title":"Sources of Data - Data Cascade","doc":"When the data is merged as part of the Data Cascade process, the order of priority for sources of data is (from highest priority to lowest):\n\n1. Front Matter Data in a Template\n1. Template Data Files *[work in progress]*\n1. Directory Data Files (and ascending Parent Directories) *[work in progress]*\n1. Front Matter Data in Layouts\n1. Configuration Global Data *[work in progress]*\n1. Global Data Files","ref":"data_cascade.html#sources-of-data"},{"type":"extras","title":"Build Process","doc":"# Build Process\n\nGriffin's `mix grf.build` task consists of a build process involving 9 separate but simple stages.\nThis document details each stage of the build process.","ref":"build_process.html"},{"type":"extras","title":"1. Call `before` hooks - Build Process","doc":"Griffin has built in support for hooks, which are arbitrary functions that can be called `before` or `after` all of the other build stages.\nAdding a hook can be done via `Application.put_env/4` or via the configuration file:\n\n```elixir\nimport Config\n\nconfig :griffin_ssg,\n  hooks: %{\n    before: [\n      fn {directories, _run_mode, _output_mode} ->\n        # compile assets, assumes esbuild is a dependency of the project\n        Mix.Task.run(\"esbuild default #{directories.data} --bundle --minify --outdir=src/assets/\")\n      end\n    ]\n  }\n```\n\n> #### Hooks are called every time the build process is called {: .info}\n>\n> Hook functions are called at the start of each stand-alone build, but it may be called multiple times if using the `--watch` command line option and file changes are detected.","ref":"build_process.html#1-call-before-hooks"},{"type":"extras","title":"2. Import global data - Build Process","doc":"This step consists of searching a specific directory for data (defaults to `data`).\nData pulled in from files this directory will be globally available for all templates and layouts to use.\nGriffin expects each file to be an Elixir script (`.exs`) where the last expression in the file is a map.\n\nAs an example, let's assume that we're writing a `metadata.exs` file with the following contents:\n\n```elixir\n%{\n  author: %{\n    email: \"geralt@example.com\",\n    name: \"Geralt of Rivia\",\n    url: \"https://example.com/about-me/\"\n  },\n  description: \"I am a legendary witcher from the School of the Wolf\",\n  language: \"en\",\n  title: \"My awesome Griffin website\",\n  url: \"https://example.com/\"\n}\n```\n\nThis data will then be accessible via the `@metadata` variable in templates.\nFor example, a layout could reference the author name in a layout:\n\n```html\n <%= @metadata.author.name %> \n```\n\nGriffin chooses the variable name to inject in templates from the filename of the data file.\nIn the previous example, if the filename was `site.exs`, the contents of the file would be available under the `@site` template variable.\n\n> #### Variables may be overriden {: .warning}\n>\n> Before generating a page, Griffin merges all available data for a template and this process may override some variables.\n> Global data has the lowest priority in the merge process. See [Data Cascade](/data_cascade.html) for more information.","ref":"build_process.html#2-import-global-data"},{"type":"extras","title":"3. Copy passthrough files - Build Process","doc":"Passthrough files are files that live in a particular directory that should simply be copied to the output path without processing.\nThis feature can be used via the `--passthrough-copies` command line option, or via the configuration file:\n\n```elixir\nimport Config\n\nconfig :griffin_ssg,\n  passthrough_copies: [\"assets/**/*.js\", \"**/*.{png,jpeg}\"]\n```\n\nThe configuration above will copy all PNG and JPEG images along with all Javascript files inside the `assets` directory.\nA file that lives in the ` /assets` will be copied to ` /assets`.","ref":"build_process.html#3-copy-passthrough-files"},{"type":"extras","title":"4. Compile layouts - Build Process","doc":"Layouts are `EEx` templates that by default live in the `lib/layouts` directory.\nThis stage of the build process consists on fetching all layouts and partials, compiling them into Elixir AST and storing them in [`:ets`](https://elixirschool.com/en/lessons/storage/ets).","ref":"build_process.html#4-compile-layouts"},{"type":"extras","title":"5. Parse files - Build Process","doc":"This stage consists of calculating the list of files that will generate HTML pages and then parsing them into a `{front_matter, content}` pair.\nSome metadata is added to the front matter, like the URL of the file and the output path.\n\nNote that the parsed files are kept in memory and are not written to disk (yet).","ref":"build_process.html#5-parse-files"},{"type":"extras","title":"6. Compile collections - Build Process","doc":"Using a list of parsed files from the previous stage, goes over the list of files and compiles a list of collections, storing information about which\npage is part of each collection.\n\nAs an example, in a blog website declaring a `tags` collection, Griffin will group all pages by their front matter value of `tags`.\nMultiple collections can be declared, enabling websites that group content by `author`, `year`, `genre`, etc.\n\nHere is an example configuration declaring `tags` as a collection:\n\n```elixir\nimport Config\n\nconfig :griffin_ssg,\n  collections: %{\n    tags: %{\n      list_layout: \"list_tags\",\n      show_layout: \"show_tags\"\n    }\n  }\n```","ref":"build_process.html#6-compile-collections"},{"type":"extras","title":"7. Render files - Build Process","doc":"This is the stage where the parsed files are actually rendered and written to disk. At this point, Griffin merges all of the available data for a template according to the [Data Cascade](/data_cascade.html).\n\nIf Griffin is running with the `--dry-run` command line option, no files will be written.","ref":"build_process.html#7-render-files"},{"type":"extras","title":"8. Render collection pages - Build Process","doc":"This stage generates multiple pages according to the number of declared collections. For a website that only declares a `tags` collection, the following pages could be generated:\n\n```text\n/tags/\n/tags/tag1/\n/tags/tag2/\n...\n```\n\nUsing this example, Griffin will use the layout referenced in the `list_layout` configuration key to render the `/tags/` page and the layout from the `show_layout` configuration key to render each of the `/tags/ ` pages. The configuration for both layouts can be set in the configuration file:\n\n```elixir\nconfig :griffin_ssg,\n  collections: %{\n    tags: %{\n      list_layout: \"list_tags\",\n      show_layout: \"show_tags\"\n    }\n  }\n```","ref":"build_process.html#8-render-collection-pages"},{"type":"extras","title":"Collection list layout - Build Process","doc":"The collection list layout (`list_layout`) is the layout used to list all of the values of a single collection. When rendering this layout, Griffin injects the following variables:\n\n* `@collection_name`, containing the name of the collection (e.g. `:tags`)\n* `@collection_values`, containing a mapping of all pages associated with each collection value:\n```elixir\n%{\n    \"tag1\" => [%{url: \"/page1/\"}, %{url: \"/page2/\"}],\n    \"tag2\" => [%{url: \"/page2/\"}]\n}\n```\n\nHere's an example of a basic list layout built for listing `tags`:\n\n```html\n<!DOCTYPE html>\n \n   \n     Tags \n   \n   \n     All Tags \n     \n    <%= for {tag, _pages} <- @collection_values do %>\n        <%= tag %>  \n    <% end %>\n     \n   \n \n```","ref":"build_process.html#collection-list-layout"},{"type":"extras","title":"Collection show layout - Build Process","doc":"The collection show layout (`show_layout`) is the layout used to list content pages associated with a single collection value. When rendering this layout, Griffin injects the following variables:\n* `@collection_name`, containing the name of the collection (e.g. `:tags`)\n* `@collection_value`, containing the name of the collection value (e.g. `\"tag1\"`)\n* `@collection_value_pages`, containing a list of maps (e.g. `[%{url: \"/page1/\"}]`),\nwhere each map contains page information such as URL, title, description, date, input and output paths. Of these attributes, title and description can be `nil` if they are not set in the page's front matter.\n\nHere's an example of a basic show layout built for listing all pages with a given `tag`:\n\n```html\n<!DOCTYPE html>\n \n   \n     Pages tagged <%= @collection_value %> \n   \n   \n     Pages tagged `<%= @collection_value %>` \n     \n    <%= for page <- @collection_value_pages do %>\n        <%= page.data.title %>  \n    <% end %>\n     \n   \n \n```\n\nNo collections pages will be written to disk if Griffin is running with the `--dry-run` command line option.","ref":"build_process.html#collection-show-layout"},{"type":"extras","title":"9. Call `after` hooks - Build Process","doc":"Similarly to the `before` hooks, Griffin supports hooks that are called after the build process is complete. Hooks can be defined via application environment or via configuration file:\n\n```elixir\nconfig :griffin_ssg,\n  hooks: %{\n    after: [\n      fn {directories, results, _run_mode, _output_mode} ->\n        # results contains a list of generated HTML files\n        Tesla.post(\n          \"https://example.com/webhooks\",\n          Jason.encode!(%{\"Files Written\" => length(results), \"success\" => true}),\n          headers: [{\"content-type\", \"application/json\"}]\n        )\n      end\n    ]\n  }\n```","ref":"build_process.html#9-call-after-hooks"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}